#!/usr/bin/perl

# uporaba: convert_ldap.pl
# skripta procita konfiguraciju iz /etc/aosi
# spoji se na imenik i u njemu obavi potrebne izmjene
# i uskladi ga sa novom shemom (v 1.3)

BEGIN {
	unshift(@INC,'/etc/aosi');
}

#use warnings;
use Net::LDAP;
use utf8;
use Digest::MD5 qw(md5_hex);
use config;

if($ARGV[0] eq '-d'){
	$debug=1;
}

#LDAP info
($LDAPip,$LDAPport)=split(/:/,$config::ldap_server);
$rtDN=$config::ldap_user;
$rtPWD=_get_pwd($config::ldap_pwd_file);
$base_dn=$config::base_dn;

unless ($ldap = Net::LDAP->new($LDAPip, port => $LDAPport, version => 'LDAPv3')){
	errorlog("Can't contact LDAP. Exiting."); 
	die();
}
unless ($mesg = $ldap->bind($rtDN, version=>3, password => $rtPWD)){  
    errorlog("Can't bind. Exiting."); 
    die();
}
$mesg = $ldap->search(
                       base   => $base_dn,
                       filter => "objectclass=*",
                       attrs   =>  []
                      );
my @entries = $mesg->entries;
foreach $entry ( @entries ) {
	my $update=1;
	if($debug) {print $entry->dn();}
	my $person=1;
	my $objclass=$entry->get_value('objectClass',asref => 1);
	foreach my $class (@$objclass){
		if($class eq 'organization'){
			$person=0;
		}
	}
	
	if($person==0){
		$entry->add ( 'hrEduOrgOIB' =>  find_oib($entry->get_value('hrEduOrgUniqueNumber',asref => 1)) );
		$entry->add ( 'objectClass' => 'schacContactLocation' );
		if(defined($hrEduOrgType=$entry->get_value('hrEduOrgType'))){
			if($hrEduOrgType =~ /(^[Ss]amostalna visoka .+kola$|^[Vv]isoka u.+iteljska .+kola$|^Javni institut$|^[Oo]stalo$|^[Uu].+iteljska akademija$|^[Oo]stalo$)/){
				$entry->replace ( 'hrEduOrgType' => ['Druge pravne osobe']);
				$_=$hrEduOrgType;
				utf8::decode($_);
				tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
				errorlog("UPOZORENJE: u org zapisu vase ustanove vrijednost atributa hrEduOrgType (tip ustanove) je : $_.\nKako po naputku MZOS-a, a u skladu s novom shemom, ta vrijednost vise ne postoji, migracijom vrijednost tog atributa je\npostavljena u Druge pravne osobe.\nMolimo da putem AOSI web sucelja (opcija: Azuriranje org zapisa) postavite ispravan tip ustanove svojoj ustanovi.\n");
			}
		}
	} else {
		my $uid=$entry->get_value('uid');
		if($uid){
			#person
			$entry->add ( 'objectClass' => 'schacContactLocation' );
			$_=$entry->get_value('cn');
			utf8::decode($_);
			tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
			$entry->add ( 
							'hrEduPersonPersistentID' => md5_hex($entry->get_value('hrEduPersonUniqueID') . "." . time),
							'hrEduPersonOIB' => find_oib($entry->get_value('hrEduPersonUniqueNumber',asref => 1)),
							'displayName' => $_);
			$displayName=$_;
			
			if(defined($hrEduPersonScienceArea=$entry->get_value('hrEduPersonScienceArea'))){
				$entry->delete ( 'hrEduPersonScienceArea' => [ ]);
				errorlog("UPOZORENJE: korisnik $uid ($displayName) ima postavljenu vrijednost\natributa hrEduPersonScienceArea (podrucje znanosti): $hrEduPersonScienceArea.\nKako je u skladu s novom shemom sifrarnik toga atributa promijenjen,\nmigracijom vrijednost tog atributa nece biti prenesena, te ju je\npotrebno azurirati u LDAP imeniku putem AOSI web sucelja.\n");
			}
			if(defined($hrEduPersonProfessionalStatus=$entry->get_value('hrEduPersonProfessionalStatus'))){
				if($hrEduPersonProfessionalStatus eq 'MR'){
					$entry->replace ( 'hrEduPersonProfessionalStatus' => ['mr.sc.']);
				} elsif($hrEduPersonProfessionalStatus eq 'DR'){
					$entry->replace ( 'hrEduPersonProfessionalStatus' => ['dr.sc.']);
				}
			}
			if(defined($hrEduPersonAcademicStatus=$entry->get_value('hrEduPersonAcademicStatus'))){
				if($hrEduPersonAcademicStatus =~ /(^mla.+i asistent$|^asistent visoke .+kole$|^znanstveni asistent$)/){
					$entry->delete ( 'hrEduPersonAcademicStatus' => [ ]);
					$_=$hrEduPersonAcademicStatus;
					utf8::decode($_);
					tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
					errorlog("UPOZORENJE: korisnik $uid ($displayName) ima postavljenu vrijednost atributa hrEduPersonAcademicStatus (zvanje): $_.\nKako u skladu s novom shemom u novom sifrarniku ta vrijednost vise ne postoji, migracijom vrijednost tog atributa nece biti\nprenesena, te ju je potrebno azurirati u LDAP imeniku putem AOSI web sucelja.\n");
				}
			}
			if(defined($hrEduPersonStudentCategory=$entry->get_value('hrEduPersonStudentCategory'))){
			#	$entry->delete ( 'hrEduPersonStudentCategory' => [ ]);
				errorlog("UPOZORENJE: korisnik $uid ($displayName) ima postavljenu vrijednost\natributa hrEduPersonStudentCategory (Vrsta studenta): $hrEduPersonStudentCategory.\nKako je u skladu s novom shemom sifrarnik toga atributa promijenjen,\nvrijednost tog atributa prenesena migracijom nece biti u skladu s novom\nshemom, pa Vas molimo da ju azurirate u LDAP imeniku putem AOSI web sucelja.\n");
			}
		} else {
			$update=0;
		}
	}
	if($update==1){
		$mesg = $entry->update($ldap);  ## THE ACTION LINE
		if($mesg->is_error()){
			if($debug) {print ": err\n";}
	  		errorlog($entry->dn() . ": " . $mesg->error());
		} else {
			if($debug) {
				if($mesg->done()){
					print ":ok\n";
				} else {
					print ":not\n";
				}
			}
		}
	}
	
}
$ldap->unbind();
print "-- finished\n";

sub errorlog {
	my ($err_mesg)=@_;
	print STDERR "$err_mesg\n";
}

sub find_oib {
	my ($arr_ref)=@_;
	my $oib='';
	foreach my $el (@$arr_ref){
		if($el =~ /^OIB: ([0-9]*)$/){
			$oib="$1";
		}
	}
	if($oib eq ''){
		$oib='NONE';
	}
	return $oib;
}
sub _get_pwd {
    my ($file) = @_;
    my ($real);

    open(FILE, "$file") or die "$0: can not open file $file for reading ($!)\n";
    $real = <FILE>;
    close(FILE);
    chomp($real);
    return $real;
}
