#!/usr/bin/perl

# uporaba: convert_ldap2.pl
# skripta procita konfiguraciju iz /etc/aosi
# spoji se na imenik i u njemu obavi potrebne izmjene
# za sukladjivanje s novom shemom koje nije obavila 
# prva skripta

BEGIN {
	unshift(@INC,'/etc/aosi');
}

#use warnings;
use Net::LDAP;
use utf8;
use Digest::MD5 qw(md5_hex);
use config;

if($ARGV[0] eq '-d'){
	$debug=1;
}

#LDAP info
($LDAPip,$LDAPport)=split(/:/,$config::ldap_server);
$rtDN=$config::ldap_user;
$rtPWD=_get_pwd($config::ldap_pwd_file);
$base_dn=$config::base_dn;

unless ($ldap = Net::LDAP->new($LDAPip, port => $LDAPport, version => 'LDAPv3')){
	errorlog("Can't contact LDAP. Exiting."); 
	die();
}
unless ($mesg = $ldap->bind($rtDN, version=>3, password => $rtPWD)){  
    errorlog("Can't bind. Exiting."); 
    die();
}

#prvo prilagodimo org zapis
my $org_data;
$mesg = $ldap->search(
           base   => $base_dn,
           filter => "objectclass=hrEduOrg",
           attrs   =>  []
          );
$mesg->code && errorlog("GRESKA: Nedostupan org zapis:" . $mesg->error."\n");
@entries = $mesg->entries;
foreach my $entry ( @entries ) {
	my $hrEduOrgType=$entry->get_value('hrEduOrgType');
	if($debug) {print $entry->dn() . ": ";}
	if($hrEduOrgType =~ /(^[Ss]amostalna visoka .+kola$|^[Vv]isoka u.+iteljska .+kola$|^Javni institut$|^[Oo]stalo$|^[Uu].+iteljska akademija$|^[Oo]stalo$)/){
		$entry->replace ( 'hrEduOrgType' => ['Druge pravne osobe']);
		$_=$hrEduOrgType;
		utf8::decode($_);
		tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
		errorlog("UPOZORENJE: u org zapisu vase ustanove vrijednost atributa hrEduOrgType\n(tip ustanove) je : $_. Kako po naputku MZOS-a, a u skladu s novom\nshemom, ta vrijednost vise ne postoji, migracijom vrijednost tog atributa\nje postavljena u Druge pravne osobe. Molimo da putem AOSI web sucelja (opcija:\nAzuriranje org zapisa) postavite ispravan tip ustanove svojoj ustanovi.\n");
		$mesg = $entry->update($ldap);  ## THE ACTION LINE
		if($mesg->is_error()){
			if($debug) {print ": err\n";}
	  		errorlog($entry->dn() . ": " . $mesg->error());
		} else {
			if($debug) {
				if($mesg->done()){
					print ":ok\n";
				} else {
					print ":not\n";
				}
			}
		}
	}
}

# azuriranje korisnika koji imaju postavljen hrEduPersonProfessionalStatus=MR ili hrEduPersonProfessionalStatus=DR
$mesg = $ldap->search(
           base   => $base_dn,
           filter => "(|(hrEduPersonProfessionalStatus=MR)(hrEduPersonProfessionalStatus=DR))",
           attrs   =>  ['uid','hrEduPersonProfessionalStatus']
          );
$mesg->code && errorlog($realm . "GRESKA: u dohvatu korisnika s krivim hrEduPersonProfessionalStatus:" . $mesg->error."\n");
if($mesg->count()>0){
	@entries = $mesg->entries;
	foreach my $entry ( @entries ) {
		my $hrEduPersonProfessionalStatus=$entry->get_value('hrEduPersonProfessionalStatus');
		if($debug) {print $entry->dn() . ": ";}
		if($hrEduPersonProfessionalStatus eq 'DR'){
			$entry->replace ( 'hrEduPersonProfessionalStatus' => ['dr.sc.']);
		} elsif($hrEduPersonProfessionalStatus eq 'MR'){
			$entry->replace ( 'hrEduPersonProfessionalStatus' => ['mr.sc.']);
		}
		$mesg = $entry->update($ldap);  ## THE ACTION LINE
		if($mesg->is_error()){
			if($debug) {print ": err\n";}
	  		errorlog($entry->dn() . ": " . $mesg->error());
		} else {
			if($debug) {
				if($mesg->done()){
					print ":ok\n";
				} else {
					print ":not\n";
				}
			}
		}
	}
}
# azuriranje korisnika koji imaju postavljen hrEduPersonAcademicStatus=mladji asistent ili hrEduPersonAcademicStatus=asistent visoke skole ili hrEduPersonAcademicStatus=znanstveni asistent
$mesg = $ldap->search(
           base   => $base_dn,
           filter => "(|(hrEduPersonAcademicStatus=mla*i asistent)(hrEduPersonAcademicStatus=asistent visoke *kole)(hrEduPersonAcademicStatus=znanstveni asistent))",
           attrs   =>  ['uid','displayName', 'hrEduPersonAcademicStatus']
          );
$mesg->code && errorlog($realm . "GRESKA: u dohvatu korisnika s krivim hrEduPersonAcademicStatus:" . $mesg->error."\n");
if($mesg->count()>0){
	@entries = $mesg->entries;
	foreach my $entry ( @entries ) {
		my $hrEduPersonAcademicStatus=$entry->get_value('hrEduPersonAcademicStatus');
		if($debug) {print $entry->dn() . ": ";}
		if($hrEduPersonAcademicStatus =~ /(^mla.+i asistent$|^asistent visoke .+kole$|^znanstveni asistent$)/){
			my $uid=$entry->get_value('uid');
			my $displayName=$entry->get_value('displayName');
			$entry->delete ( 'hrEduPersonAcademicStatus' => [ ]);
			$_=$hrEduPersonAcademicStatus;
			utf8::decode($_);
			tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
			errorlog("UPOZORENJE: korisnik $uid ($displayName) ima postavljenu vrijednost\natributa hrEduPersonAcademicStatus (zvanje): $_. Kako u skladu snovom shemom u novom sifrarniku ta vrijednost vise ne postoji, migracijom\nvrijednost tog atributa nece biti prenesena, te ju je potrebno azurirati\nu LDAP imeniku putem AOSI web sucelja.\n");
			$mesg = $entry->update($ldap);  ## THE ACTION LINE
			if($mesg->is_error()){
				if($debug) {print ": err\n";}
		  		errorlog($entry->dn() . ": " . $mesg->error());
			} else {
				if($debug) {
					if($mesg->done()){
						print ":ok\n";
					} else {
						print ":not\n";
					}
				}
			}
		}
	}
}

# za korisnike koji imaju postavljenu vrijednost atributa hrEduPersonStudentCategory koja nije u skladu sa novim sifrarnikom ispisuje se upozorenje, ali nema azuriranja u imeniku.
#te administratore upucujemo da nam se jave kako bi im mi pomogli azurirati imenik (ocekujemo da se na ustanovama koje postavljaju vrijednost tog atributa nadje velik broj zapisa s neispravnom vrijednosti)
$mesg = $ldap->search(
           base   => $base_dn,
           filter => "(hrEduPersonStudentCategory=*)",
           attrs   =>  ['uid','displayName', 'hrEduPersonStudentCategory']
          );
$mesg->code && errorlog($realm . "GRESKA: u dohvatu korisnika s krivim hrEduPersonStudentCategory:" . $mesg->error."\n");
if($mesg->count()>0){
	@entries = $mesg->entries;
	my $krivi_studenti='';
	my $broj_krivih=0;
	foreach my $entry ( @entries ) {
		my $hrEduPersonStudentCategory=$entry->get_value('hrEduPersonStudentCategory');
		if($debug) {print $entry->dn() . ": ";}
# ako je upisao pauzu ili je studij u prekidu to se mijenja u mirovanje statusa studenta
		if($hrEduPersonStudentCategory eq 'pauzira godinu' || $hrEduPersonStudentCategory eq 'prekid studija'){
			$entry->replace ( 'hrEduPersonStudentCategory' => ['mirovanje statusa studenta']);
			$mesg = $entry->update($ldap);  ## THE ACTION LINE
			if($mesg->is_error()){
				if($debug) {print ": err\n";}
		  		errorlog($entry->dn() . ": " . $mesg->error());
			} else {
				if($debug) {
					if($mesg->done()){
						print ":ok\n";
					} else {
						print ":not\n";
					}
				}
			}
#ako je srednjoskolac ili osnovnoskolac, sve je ok
		} elsif ($hrEduPersonStudentCategory =~ /(^srednjo.+kolac$|^osnovno.+kolac$)/){
			if($debug) {print ":ok\n";}
		} else {
			if($debug) {print ":kriva vrsta studenta\n";}
			my $uid=$entry->get_value('uid');
			my $displayName=$entry->get_value('displayName');
			$_=$hrEduPersonStudentCategory;
			utf8::decode($_);
			tr /\x{0106}\x{0107}\x{010C}\x{010D}\x{0110}\x{0111}\x{0160}\x{0161}\x{017D}\x{017E}/CcCcDdSsZz/;
			$krivi_studenti.="$uid ($displayName): $_\n";
			$broj_krivih++;
		}
	}
	if($broj_krivih > 0){
		errorlog("UPOZORENJE: u vasem imeniku $broj_krivih studenata ima postavljenu\nvrijednost atributa hrEduPersonStudentCategory (Vrsta studenta) koja nije\nu skladu s novim sifrarnikom. U nastavku slijedi popis takvih studenata.\nSvim studentima s popisa potrebno je uskladiti  vrijednost tog atributa s\nnovim sifrarnikom. Ukoliko se radi o velikom broju zapisa, molimo da nam\nse javite na adresu team@aaiedu.hr kako bi vam pomogli azurirati vas\nimenik.\n-------Popis------\n$krivi_studenti ----------\n");
	}
}

$ldap->unbind();
print "-- migration is finished\n";

sub errorlog {
	my ($err_mesg)=@_;
	print STDERR "$err_mesg\n";
}

sub _get_pwd {
    my ($file) = @_;
    my ($real);

    open(FILE, "$file") or die "$0: can not open file $file for reading ($!)\n";
    $real = <FILE>;
    close(FILE);
    chomp($real);
    return $real;
}
